@@ -17,19 +17,17 @@
 
 #include "EntryModel.h"
 
-#include <QDateTime>
 #include <QFont>
 #include <QMimeData>
-#include <QPainter>
 #include <QPalette>
 
-#include "core/Config.h"
-#include "core/DatabaseIcons.h"
 #include "core/Entry.h"
-#include "core/Global.h"
 #include "core/Group.h"
 #include "core/Metadata.h"
-#include "core/Resources.h"
+#include "core/PasswordHealth.h"
+#include "gui/DatabaseIcons.h"
+#include "gui/Icons.h"
+#include "gui/styles/StateColorPalette.h"
 #ifdef Q_OS_MACOS
 #include "gui/osutils/macutils/MacUtils.h"
 #endif
@@ -37,11 +35,10 @@
 EntryModel::EntryModel(QObject* parent)
     : QAbstractTableModel(parent)
     , m_group(nullptr)
-    , m_hideUsernames(false)
-    , m_hidePasswords(true)
     , HiddenContentDisplay(QString("\u25cf").repeated(6))
     , DateFormat(Qt::DefaultLocaleShortDate)
 {
+    connect(config(), &Config::changed, this, &EntryModel::onConfigChanged);
 }
 
 Entry* EntryModel::entryFromIndex(const QModelIndex& index) const
@@ -53,8 +50,10 @@ Entry* EntryModel::entryFromIndex(const QModelIndex& index) const
 QModelIndex EntryModel::indexFromEntry(Entry* entry) const
 {
     int row = m_entries.indexOf(entry);
-    Q_ASSERT(row != -1);
-    return index(row, 1);
+    if (row >= 0) {
+        return index(row, 1);
+    }
+    return {};
 }
 
 void EntryModel::setGroup(Group* group)
@@ -88,25 +87,13 @@ void EntryModel::setEntries(const QList<Entry*>& entries)
     m_entries = entries;
     m_orgEntries = entries;
 
-    QSet<Database*> databases;
-
-    for (Entry* entry : asConst(m_entries)) {
-        databases.insert(entry->group()->database());
-    }
-
-    for (Database* db : asConst(databases)) {
-        Q_ASSERT(db);
-        const QList<Group*> groupList = db->rootGroup()->groupsRecursive(true);
-        for (const Group* group : groupList) {
-            m_allGroups.append(group);
-        }
-
-        if (db->metadata()->recycleBin()) {
-            m_allGroups.removeOne(db->metadata()->recycleBin());
+    for (const auto entry : asConst(m_entries)) {
+        if (entry->group()) {
+            m_allGroups.insert(entry->group());
         }
     }
 
-    for (const Group* group : asConst(m_allGroups)) {
+    for (const auto group : m_allGroups) {
         makeConnections(group);
     }
 
@@ -129,7 +116,7 @@ int EntryModel::columnCount(const QModelIndex& parent) const
         return 0;
     }
 
-    return 14;
+    return 17;
 }
 
 QVariant EntryModel::data(const QModelIndex& index, int role) const
@@ -149,6 +136,11 @@ QVariant EntryModel::data(const QModelIndex& index, int role) const
                 return entry->group()->name();
             }
             break;
+        case ParentGroupPath:
+            if (entry->group()) {
+                return entry->group()->fullPath();
+            }
+            break;
         case Title:
             result = entry->resolveMultiplePlaceholders(entry->title());
             if (attr->isReference(EntryAttributes::TitleKey)) {
@@ -156,7 +148,7 @@ QVariant EntryModel::data(const QModelIndex& index, int role) const
             }
             return result;
         case Username:
-            if (m_hideUsernames) {
+            if (config()->get(Config::GUI_HideUsernames).toBool()) {
                 result = EntryModel::HiddenContentDisplay;
             } else {
                 result = entry->resolveMultiplePlaceholders(entry->username());
@@ -164,9 +156,12 @@ QVariant EntryModel::data(const QModelIndex& index, int role) const
             if (attr->isReference(EntryAttributes::UserNameKey)) {
                 result.prepend(tr("Ref: ", "Reference abbreviation"));
             }
+            if (entry->username().isEmpty() && !config()->get(Config::Security_PasswordEmptyPlaceholder).toBool()) {
+                result = "";
+            }
             return result;
         case Password:
-            if (m_hidePasswords) {
+            if (config()->get(Config::GUI_HidePasswords).toBool()) {
                 result = EntryModel::HiddenContentDisplay;
             } else {
                 result = entry->resolveMultiplePlaceholders(entry->password());
@@ -240,6 +235,13 @@ QVariant EntryModel::data(const QModelIndex& index, int role) const
 
             return result;
         }
+        case Color:
+            QColor backgroundColor;
+            backgroundColor.setNamedColor(entry->backgroundColor());
+            if (backgroundColor.isValid()) {
+                result = "▍";
+                return result;
+            }
         }
     } else if (role == Qt::UserRole) { // Qt::UserRole is used as sort role, see EntryView::EntryView()
         switch (index.column()) {
@@ -247,6 +249,12 @@ QVariant EntryModel::data(const QModelIndex& index, int role) const
             return entry->resolveMultiplePlaceholders(entry->username());
         case Password:
             return entry->resolveMultiplePlaceholders(entry->password());
+        case PasswordStrength: {
+            if (!entry->password().isEmpty() && !entry->excludeFromReports()) {
+                return entry->passwordHealth()->score();
+            }
+            return 0;
+        }
         case Expires:
             // There seems to be no better way of expressing 'infinity'
             return entry->timeInfo().expires() ? entry->timeInfo().expiryTime() : QDateTime(QDate(9999, 1, 1));
@@ -273,19 +281,47 @@ QVariant EntryModel::data(const QModelIndex& index, int role) const
         switch (index.column()) {
         case ParentGroup:
             if (entry->group()) {
-                return entry->group()->iconPixmap();
+                return Icons::groupIconPixmap(entry->group());
             }
             break;
         case Title:
-            return entry->iconPixmap();
+            return Icons::entryIconPixmap(entry);
         case Paperclip:
             if (!entry->attachments()->isEmpty()) {
-                return resources()->icon("paperclip");
+                return icons()->icon("paperclip");
             }
             break;
         case Totp:
             if (entry->hasTotp()) {
-                return resources()->icon("chronometer");
+                return icons()->icon("totp");
+            }
+            break;
+        case PasswordStrength:
+            if (!entry->password().isEmpty() && !entry->excludeFromReports()) {
+                QString iconName = "lock-question";
+                StateColorPalette statePalette;
+                QColor color = statePalette.color(StateColorPalette::Error);
+
+                switch (entry->passwordHealth()->quality()) {
+                case PasswordHealth::Quality::Bad:
+                case PasswordHealth::Quality::Poor:
+                    iconName = "lock-open-alert";
+                    color = statePalette.color(StateColorPalette::HealthCritical);
+                    break;
+                case PasswordHealth::Quality::Weak:
+                    iconName = "lock-open";
+                    color = statePalette.color(StateColorPalette::HealthBad);
+                    break;
+                case PasswordHealth::Quality::Good:
+                case PasswordHealth::Quality::Excellent:
+                    iconName = "lock";
+                    color = statePalette.color(StateColorPalette::HealthExcellent);
+                    break;
+                }
+
+                if (color.isValid()) {
+                    return icons()->icon(iconName, true, color);
+                }
             }
             break;
         }
@@ -296,6 +332,15 @@ QVariant EntryModel::data(const QModelIndex& index, int role) const
         }
         return font;
     } else if (role == Qt::ForegroundRole) {
+
+        if (index.column() == Color) {
+            QColor backgroundColor;
+            backgroundColor.setNamedColor(entry->backgroundColor());
+            if (backgroundColor.isValid()) {
+                return backgroundColor;
+            }
+        }
+
         QColor foregroundColor;
         foregroundColor.setNamedColor(entry->foregroundColor());
         if (entry->hasReferences()) {
@@ -309,14 +354,16 @@ QVariant EntryModel::data(const QModelIndex& index, int role) const
             return QVariant(foregroundColor);
         }
     } else if (role == Qt::BackgroundRole) {
-        QColor backgroundColor;
-        backgroundColor.setNamedColor(entry->backgroundColor());
-        if (backgroundColor.isValid()) {
-            return QVariant(backgroundColor);
+        if (m_backgroundColorVisible) {
+            QColor backgroundColor;
+            backgroundColor.setNamedColor(entry->backgroundColor());
+            if (backgroundColor.isValid()) {
+                return QVariant(backgroundColor);
+            }
         }
-    } else if (role == Qt::TextAlignmentRole) {
-        if (index.column() == Paperclip) {
-            return Qt::AlignCenter;
+    } else if (role == Qt::ToolTipRole) {
+        if (index.column() == PasswordStrength && !entry->password().isEmpty() && !entry->excludeFromReports()) {
+            return entry->passwordHealth()->scoreReason();
         }
     }
 
@@ -331,6 +378,8 @@ QVariant EntryModel::headerData(int section, Qt::Orientation orientation, int ro
         switch (section) {
         case ParentGroup:
             return tr("Group");
+        case ParentGroupPath:
+            return tr("Group Path");
         case Title:
             return tr("Title");
         case Username:
@@ -358,20 +407,26 @@ QVariant EntryModel::headerData(int section, Qt::Orientation orientation, int ro
     } else if (role == Qt::DecorationRole) {
         switch (section) {
         case Paperclip:
-            return resources()->icon("paperclip");
+            return icons()->icon("paperclip");
         case Totp:
-            return resources()->icon("chronometer");
+            return icons()->icon("totp");
+        case PasswordStrength:
+            return icons()->icon("lock-question");
         }
     } else if (role == Qt::ToolTipRole) {
         switch (section) {
         case ParentGroup:
             return tr("Group name");
+        case ParentGroupPath:
+            return tr("Group Path");
         case Title:
             return tr("Entry title");
         case Username:
             return tr("Username");
         case Password:
             return tr("Password");
+        case PasswordStrength:
+            return tr("Password Strength");
         case Url:
             return tr("URL");
         case Notes:
@@ -391,7 +446,9 @@ QVariant EntryModel::headerData(int section, Qt::Orientation orientation, int ro
         case Paperclip:
             return tr("Has attachments");
         case Totp:
-            return tr("Has TOTP one-time password");
+            return tr("Has TOTP");
+        case Color:
+            return tr("Background Color");
         }
     }
 
@@ -537,6 +594,20 @@ void EntryModel::entryDataChanged(Entry* entry)
     emit dataChanged(index(row, 0), index(row, columnCount() - 1));
 }
 
+void EntryModel::onConfigChanged(Config::ConfigKey key)
+{
+    switch (key) {
+    case Config::GUI_HideUsernames:
+        emit dataChanged(index(0, Username), index(rowCount() - 1, Username), {Qt::DisplayRole});
+        break;
+    case Config::GUI_HidePasswords:
+        emit dataChanged(index(0, Password), index(rowCount() - 1, Password), {Qt::DisplayRole});
+        break;
+    default:
+        break;
+    }
+}
+
 void EntryModel::severConnections()
 {
     if (m_group) {
@@ -560,39 +631,7 @@ void EntryModel::makeConnections(const Group* group)
     connect(group, SIGNAL(entryMovedDown()), SLOT(entryMovedDown()));
     connect(group, SIGNAL(entryDataChanged(Entry*)), SLOT(entryDataChanged(Entry*)));
 }
-
-/**
- * Get current state of 'Hide Usernames' setting
- */
-bool EntryModel::isUsernamesHidden() const
+void EntryModel::setBackgroundColorVisible(bool visible)
 {
-    return m_hideUsernames;
-}
-
-/**
- * Set state of 'Hide Usernames' setting and signal change
- */
-void EntryModel::setUsernamesHidden(bool hide)
-{
-    m_hideUsernames = hide;
-    emit dataChanged(index(0, 0), index(rowCount() - 1, columnCount() - 1));
-    emit usernamesHiddenChanged();
-}
-
-/**
- * Get current state of 'Hide Passwords' setting
- */
-bool EntryModel::isPasswordsHidden() const
-{
-    return m_hidePasswords;
-}
-
-/**
- * Set state of 'Hide Passwords' setting and signal change
- */
-void EntryModel::setPasswordsHidden(bool hide)
-{
-    m_hidePasswords = hide;
-    emit dataChanged(index(0, 0), index(rowCount() - 1, columnCount() - 1));
-    emit passwordsHiddenChanged();
+    m_backgroundColorVisible = visible;
 }
